# This script has two methods of calculating predominance - the standard one ('ht1') based on most moles and a redox-based one 
# ('redox') that divides the species based on most moles for a particular valence state (secondary master species), e.g. U(4), U(6) etc.
# Thanks to Rémi Marsac (INE) for the suggestion.

# The <mainspecies> tag sets the main 'species', e.g. "U". Only species containing this 'master species' will be plotted.
# The <method> tag decides which one to use: set to either "ht1" or else it assumes "redox".

PHASES
Fix_H+
   H+ = H+
   log_k 0.0

PRINT
   -reset false
   
SELECTED_OUTPUT 1
   -high_precision true
   -reset false
   
USER_PUNCH 1
-headings 
-start
10 IF(STEP_NO = 0) THEN 750 ELSE 20
20 main$ = "<mainspecies>"
30 method$ = "<method>"
40 nout1 = 0
50 nout2 = 0
60 nout3 = 0
70 minc = 1e-16
80 h2o = TOT("water")
90 
100 IF(method$ = "ht1") THEN 120 ELSE 270
101
110 REM Predominance basesd on simple abundance
120 totel = SYS(main$, n, n$, t$, moles)
130 REM Predominant species based on concn (but filter out gases)
140 REM species are sorted largest first
150 FOR i = 1 TO n
160    IF(moles(i) <= minc) THEN 520
170    IF(INSTR(n$(i), "(g)") > 0) THEN 230
180    PUNCH n$(i), moles(i)/h2o
190    REM PRINT "PUNCH", i, n$(i), moles(i)/h2o
200    nout1 = nout1+1
210    IF(nout1 >= 3) THEN 520
220 NEXT i
230 GOTO 520
240 
250 
260 REM
270 n = 17
280 DIM s(n), ns$(n), indx(n)
290 REM loop through valence states involved, say -8 to +8
300 FOR j = 1 TO n
310    i = ABS(j-9)
370    ns$(j) = main$ + "(" + TRIM(STR$(i)) + ")"
380    s(j) = SYS(ns$(j), ne, n$, t$, moles)
400 NEXT j
401
410 REM sort concns
420 GOSUB 30000
421
430 REM Output predominant redox species for PhreePlot
440 FOR i = n TO 1 STEP -1
450    IF(s(i) <= minc) THEN 500
460    REM PRINT i, indx(i), ns$(indx(i)), s(i)/h2o
470    PUNCH ns$(indx(i)), s(i)/h2o
480    nout1 = nout1+1
490    IF(nout1 >= 3) THEN 520
500 NEXT i
510 REM no forced species (see ht1stability.inc)
511
520 nout2 = 0
530 REM add these constraints or overrides
540 IF(SI("H2(g)") > 0) THEN 550 ELSE 570
550 PUNCH "H2(g) > 1 atm", 10^SI("H2(g)")
560 nout3 = nout3+1
570 IF(SI("O2(g)") > -0.677) THEN 580 ELSE 600
580 PUNCH "O2(g) > 0.21 atm", 10^SI("O2(g)")
590 nout3 = nout3+1
600 IF(SI("CH4(g)") > 0) THEN 610 ELSE 630
610 PUNCH "CH4(g) > 1 atm", 10^SI("CH4(g)")
620 nout3 = nout3+1
630 REM add any extra 'carry' variables for viewing
640 nout4 = 0
650 REM the 5 system variables (always required)
660 po2 = SI("O2(g)")
670 ph2 = SI("H2(g)")
680 ph = -LA("H+")
690 pe = -LA("e-")
700 PUNCH "pH", ph, "pe", pe, "PO2", po2, "PH2", ph2, "temp", TC
710 REM output the 5 counters
720 nout5 = 5
730 PRINT "nout = ", nout1, nout2, nout3, nout4, nout5
740 PUNCH nout1, nout2, nout3, nout4, nout5
750 END


# SORT ROUTINE - input: n, s(n), indx(n)  output: s(n), indx(n)
30000 REM insertion sort with index (after Numerical Recipes)
30010 REM input: 
30020 REM   s() is a numeric array of length n to be sorted
30030 REM   indx() should start with an array of length at least n
30040 REM output:
30050 REM   s() is sorted
30060 REM   indx() contains sorted sequence        
30070 REM
30090
30100 REM set indices to 1, 2, ... n
30110 FOR js = 1 TO n
30120    indx(js) = js
30130 NEXT js
30140
30150 IF(n < 2) THEN GOTO 30280
30160 FOR js = 2 TO n
30170    as = s(js)
30180    bs = indx(js)
30190    FOR is = js-1 TO 1 STEP -1
30200       IF(s(is) <= as) THEN 30250
30210       s(is+1) = s(is)
30220       indx(is+1) = indx(is)
30230    NEXT is
30240    is = 0
30250    s(is+1) = as
30260    indx(is+1) = bs
30270 NEXT js
30280 RETURN
-end
