# Used by ht1/grid calculations to calculate a type of mineral stability diagram based on selecting the mineral
# phase with the highest SI providing it exceeds 0 (strictly greater than or equal to 'acc', a value very close to zero, namely -1e-11).
# If no mineral is saturated, then the predominant 'mainSpecies' species is plotted. Saturated minerals are not allowed to precipitate.
# N.B. The mineral species need not contain 'mainspecies'

# *** None of the mineral phases should be specified in the EQUILIBRIUM_PHASES keyword data block
PRINT
-reset FALSE
SELECTED_OUTPUT
  reset false
  high_precision true
USER_PUNCH
-start
10 acc = -1e-11 # may need adjusting
20 nout1 = 0
30 nout2 = 0
40 nout3 = 0
50 nout4 = 0
60 nout5 = 0

100 REM Default
110 PUNCH "No minerals", 0
120 nout1 = 1
130 IF(STEP_NO = 0) THEN 410

200 REM get a list of phases sorted in order of descending SI
210 t = SYS("phases", n, n$, t$, six)
220 FOR i = 1 TO n
230 REM Eliminate non-mineral phases
240   IF(INSTR(n$(i), "Fix")) > 0 THEN 330
250   IF(INSTR(n$(i), "(g)")) > 0 THEN 330
260   REM PRINT  i, PAD(n$(i),30), "SI = ", six(i), EQUI_DELTA(n$(i))
270   IF (six(i)<acc) THEN 400
280 REM Filter
290   REM IF (n$(i) = "Kaolinite" OR n$(i) = "K-feldspar" OR n$(i) = "K-mica" OR n$(i) = "Gibbsite" OR n$(i) = "Ca-Montmorillonite" OR n$(i) = "Quartz") THEN 300 ELSE 330
300   PUNCH n$(i), six(i)
310   nout1 = nout1 + 1
320   IF (nout1=3) THEN 410
330 NEXT i

400 REM now add the usual system variables
410 po2 = SI("O2(g)")
420 ph2 = SI("H2(g)")
430 ph = -LA("H+")
440 pe = -LA("e-")
450 PUNCH "pH", ph, "pe", pe, "PO2", po2, "PH2", ph2, "temp", tc
460 nout5 = 5
470 PUNCH nout1, nout2, nout3, nout4, nout5
480 END

